@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
 
 Copyright (C) 2014  Vyacheslav Trushkin
-Copyright (C) 2020-2023  Boian Bonev
+Copyright (C) 2020-2024  Boian Bonev
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 
@@ -50,7 +50,7 @@ inline int get_vm_counters(uint64_t *pgpgin,uint64_t *pgpgou) {
 			t=realloc(buf,bs+BSIZ);
 
 			if (!t) {
-				free(buf);
+				free(buf); // gcc-13 yields false positive -Wuse-after-free here
 				close(fd);
 				return ENOMEM;
 			}
